@@ -31,7 +31,7 @@
  */
 
 #include <test/support/tdb_catch.h>
-#include "test/src/helpers.h"
+#include "test/support/src/helpers.h"
 #include "tiledb/api/c_api/context/context_api_internal.h"
 #include "tiledb/sm/array/array_directory.h"
 #include "tiledb/sm/c_api/tiledb_struct_def.h"
@@ -52,6 +52,7 @@ struct ConsolidationWithTimestampsFx {
   Context ctx_;
   VFS vfs_;
   sm::StorageManager* sm_;
+  bool serialized_writes_ = false;
 
   // Constructors/destructors.
   ConsolidationWithTimestampsFx();
@@ -92,7 +93,6 @@ struct ConsolidationWithTimestampsFx {
   void remove_sparse_array();
   void remove_array(const std::string& array_name);
   bool is_array(const std::string& array_name);
-  bool serialized_writes = false;
 };
 
 ConsolidationWithTimestampsFx::ConsolidationWithTimestampsFx()
@@ -177,7 +177,7 @@ void ConsolidationWithTimestampsFx::write_sparse(
   query.set_data_buffer("d1", dim1);
   query.set_data_buffer("d2", dim2);
 
-  if (!serialized_writes) {
+  if (!serialized_writes_) {
     query.submit();
     query.finalize();
   } else {
@@ -210,7 +210,7 @@ void ConsolidationWithTimestampsFx::write_sparse_v11(uint64_t timestamp) {
   query.set_data_buffer("a3", buffer_a3);
   query.set_data_buffer("d1", buffer_coords_dim1);
   query.set_data_buffer("d2", buffer_coords_dim2);
-  if (!serialized_writes) {
+  if (!serialized_writes_) {
     query.submit();
     query.finalize();
   } else {
@@ -381,13 +381,13 @@ TEST_CASE_METHOD(
   create_sparse_array();
 
   SECTION("no serialization") {
-    serialized_writes = false;
+    serialized_writes_ = false;
   }
-  SECTION("serialization enabled global order write") {
 #ifdef TILEDB_SERIALIZATION
-    serialized_writes = true;
-#endif
+  SECTION("serialization enabled global order write") {
+    serialized_writes_ = true;
   }
+#endif
   // Write first fragment.
   write_sparse(
       {0, 1, 2, 3, 4, 5, 6, 7},
@@ -416,13 +416,13 @@ TEST_CASE_METHOD(
   create_sparse_array(true);
 
   SECTION("no serialization") {
-    serialized_writes = false;
+    serialized_writes_ = false;
   }
-  SECTION("serialization enabled global order write") {
 #ifdef TILEDB_SERIALIZATION
-    serialized_writes = true;
-#endif
+  SECTION("serialization enabled global order write") {
+    serialized_writes_ = true;
   }
+#endif
   // Write first fragment.
   write_sparse({0, 1, 2, 3}, {1, 1, 1, 2}, {1, 2, 4, 3}, 1);
 
@@ -499,13 +499,13 @@ TEST_CASE_METHOD(
   create_sparse_array();
 
   SECTION("no serialization") {
-    serialized_writes = false;
+    serialized_writes_ = false;
   }
-  SECTION("serialization enabled global order write") {
 #ifdef TILEDB_SERIALIZATION
-    serialized_writes = true;
-#endif
+  SECTION("serialization enabled global order write") {
+    serialized_writes_ = true;
   }
+#endif
   // Write first fragment.
   write_sparse(
       {0, 1, 2, 3, 4, 5, 6, 7},
@@ -555,13 +555,13 @@ TEST_CASE_METHOD(
   }
 
   SECTION("no serialization") {
-    serialized_writes = false;
+    serialized_writes_ = false;
   }
-  SECTION("serialization enabled global order write") {
 #ifdef TILEDB_SERIALIZATION
-    serialized_writes = true;
-#endif
+  SECTION("serialization enabled global order write") {
+    serialized_writes_ = true;
   }
+#endif
 
   remove_sparse_array();
   create_sparse_array_v11();
@@ -644,13 +644,13 @@ TEST_CASE_METHOD(
   create_sparse_array();
 
   SECTION("no serialization") {
-    serialized_writes = false;
+    serialized_writes_ = false;
   }
-  SECTION("serialization enabled global order write") {
 #ifdef TILEDB_SERIALIZATION
-    serialized_writes = true;
-#endif
+  SECTION("serialization enabled global order write") {
+    serialized_writes_ = true;
   }
+#endif
 
   // Write fragments.
   for (uint64_t i = 0; i < 50; i++) {
@@ -693,13 +693,13 @@ TEST_CASE_METHOD(
   create_sparse_array(allow_dups);
 
   SECTION("no serialization") {
-    serialized_writes = false;
+    serialized_writes_ = false;
   }
-  SECTION("serialization enabled global order write") {
 #ifdef TILEDB_SERIALIZATION
-    serialized_writes = true;
-#endif
+  SECTION("serialization enabled global order write") {
+    serialized_writes_ = true;
   }
+#endif
 
   // Write fragments.
   // We write 8 cells per fragments for 6 fragments. Then it gets consolidated
@@ -757,13 +757,13 @@ TEST_CASE_METHOD(
   create_sparse_array();
 
   SECTION("no serialization") {
-    serialized_writes = false;
+    serialized_writes_ = false;
   }
-  SECTION("serialization enabled global order write") {
 #ifdef TILEDB_SERIALIZATION
-    serialized_writes = true;
-#endif
+  SECTION("serialization enabled global order write") {
+    serialized_writes_ = true;
   }
+#endif
   // Write fragments.
   for (uint64_t i = 0; i < 50; i++) {
     std::vector<int> a(1);
@@ -808,13 +808,13 @@ TEST_CASE_METHOD(
   create_sparse_array();
 
   SECTION("no serialization") {
-    serialized_writes = false;
+    serialized_writes_ = false;
   }
-  SECTION("serialization enabled global order write") {
 #ifdef TILEDB_SERIALIZATION
-    serialized_writes = true;
-#endif
+  SECTION("serialization enabled global order write") {
+    serialized_writes_ = true;
   }
+#endif
   // Write fragments.
   // We write 8 cells per fragment for 6 fragments. Then it gets consolidated
   // into one. So we'll get in order 6xcell1, 6xcell2... total 48 cells. Tile
@@ -867,13 +867,13 @@ TEST_CASE_METHOD(
   create_sparse_array(true);
 
   SECTION("no serialization") {
-    serialized_writes = false;
+    serialized_writes_ = false;
   }
-  SECTION("serialization enabled global order write") {
 #ifdef TILEDB_SERIALIZATION
-    serialized_writes = true;
-#endif
+  SECTION("serialization enabled global order write") {
+    serialized_writes_ = true;
   }
+#endif
   // Write first fragment.
   write_sparse({0, 1, 2, 3}, {1, 1, 1, 2}, {1, 2, 4, 3}, 1);
 
@@ -962,13 +962,13 @@ TEST_CASE_METHOD(
   create_sparse_array();
 
   SECTION("no serialization") {
-    serialized_writes = false;
+    serialized_writes_ = false;
   }
-  SECTION("serialization enabled global order write") {
 #ifdef TILEDB_SERIALIZATION
-    serialized_writes = true;
-#endif
+  SECTION("serialization enabled global order write") {
+    serialized_writes_ = true;
   }
+#endif
   // Write first fragment.
   write_sparse({0, 1, 2, 3}, {1, 1, 1, 2}, {1, 2, 4, 3}, 1);
 
@@ -1040,13 +1040,13 @@ TEST_CASE_METHOD(
   create_sparse_array(true);
 
   SECTION("no serialization") {
-    serialized_writes = false;
+    serialized_writes_ = false;
   }
-  SECTION("serialization enabled global order write") {
 #ifdef TILEDB_SERIALIZATION
-    serialized_writes = true;
-#endif
+  SECTION("serialization enabled global order write") {
+    serialized_writes_ = true;
   }
+#endif
   // Write first fragment.
   write_sparse({0, 1, 2, 3}, {1, 1, 1, 2}, {1, 2, 4, 3}, 1);
   // Write second fragment.
@@ -1194,13 +1194,13 @@ TEST_CASE_METHOD(
   create_sparse_array(true);
 
   SECTION("no serialization") {
-    serialized_writes = false;
+    serialized_writes_ = false;
   }
-  SECTION("serialization enabled global order write") {
 #ifdef TILEDB_SERIALIZATION
-    serialized_writes = true;
-#endif
+  SECTION("serialization enabled global order write") {
+    serialized_writes_ = true;
   }
+#endif
   // Write first fragment.
   write_sparse({0, 1, 2, 3}, {1, 1, 1, 2}, {1, 2, 4, 3}, 1);
   // Write second fragment.
@@ -1310,13 +1310,13 @@ TEST_CASE_METHOD(
   create_sparse_array(true);
 
   SECTION("no serialization") {
-    serialized_writes = false;
+    serialized_writes_ = false;
   }
-  SECTION("serialization enabled global order write") {
 #ifdef TILEDB_SERIALIZATION
-    serialized_writes = true;
-#endif
+  SECTION("serialization enabled global order write") {
+    serialized_writes_ = true;
   }
+#endif
   // Write first fragment.
   write_sparse({0, 1, 2, 3}, {1, 1, 1, 2}, {1, 2, 4, 3}, 1);
   // Write second fragment.
@@ -1411,13 +1411,13 @@ TEST_CASE_METHOD(
   create_sparse_array(true);
 
   SECTION("no serialization") {
-    serialized_writes = false;
+    serialized_writes_ = false;
   }
-  SECTION("serialization enabled global order write") {
 #ifdef TILEDB_SERIALIZATION
-    serialized_writes = true;
-#endif
+  SECTION("serialization enabled global order write") {
+    serialized_writes_ = true;
   }
+#endif
   // Write first fragment.
   write_sparse({0, 1, 2, 3}, {1, 1, 1, 2}, {1, 2, 4, 3}, 1);
   // Write second fragment.
@@ -1520,13 +1520,13 @@ TEST_CASE_METHOD(
   create_sparse_array(true);
 
   SECTION("no serialization") {
-    serialized_writes = false;
+    serialized_writes_ = false;
   }
-  SECTION("serialization enabled global order write") {
 #ifdef TILEDB_SERIALIZATION
-    serialized_writes = true;
-#endif
+  SECTION("serialization enabled global order write") {
+    serialized_writes_ = true;
   }
+#endif
   // Write first fragment.
   write_sparse({0, 1, 2, 3}, {1, 1, 1, 2}, {1, 2, 4, 3}, 1);
   // Write second fragment.
@@ -1631,13 +1631,13 @@ TEST_CASE_METHOD(
   create_sparse_array(true);
 
   SECTION("no serialization") {
-    serialized_writes = false;
+    serialized_writes_ = false;
   }
-  SECTION("serialization enabled global order write") {
 #ifdef TILEDB_SERIALIZATION
-    serialized_writes = true;
-#endif
+  SECTION("serialization enabled global order write") {
+    serialized_writes_ = true;
   }
+#endif
   // Write first fragment.
   write_sparse({0, 1, 2, 3}, {1, 1, 1, 2}, {1, 2, 4, 3}, 1);
   // Write second fragment.
@@ -1720,13 +1720,13 @@ TEST_CASE_METHOD(
     create_sparse_array(false);
 
     SECTION("no serialization") {
-      serialized_writes = false;
+      serialized_writes_ = false;
     }
-    SECTION("serialization enabled global order write") {
 #ifdef TILEDB_SERIALIZATION
-      serialized_writes = true;
-#endif
+    SECTION("serialization enabled global order write") {
+      serialized_writes_ = true;
     }
+#endif
     // Write first fragment.
     write_sparse({0, 1, 2, 3}, {1, 1, 1, 2}, {1, 2, 4, 3}, 1);
     // Write second fragment.
@@ -1747,13 +1747,13 @@ TEST_CASE_METHOD(
     create_sparse_array(false);
 
     SECTION("no serialization") {
-      serialized_writes = false;
+      serialized_writes_ = false;
     }
-    SECTION("serialization enabled global order write") {
 #ifdef TILEDB_SERIALIZATION
-      serialized_writes = true;
-#endif
+    SECTION("serialization enabled global order write") {
+      serialized_writes_ = true;
     }
+#endif
     // Write first fragment.
     write_sparse({0, 1, 2, 3}, {1, 1, 1, 2}, {1, 2, 4, 3}, 1);
     // Write second fragment.
@@ -1793,13 +1793,13 @@ TEST_CASE_METHOD(
   create_sparse_array(dups);
 
   SECTION("no serialization") {
-    serialized_writes = false;
+    serialized_writes_ = false;
   }
-  SECTION("serialization enabled global order write") {
 #ifdef TILEDB_SERIALIZATION
-    serialized_writes = true;
-#endif
+  SECTION("serialization enabled global order write") {
+    serialized_writes_ = true;
   }
+#endif
   // Write first fragment.
   write_sparse({0, 1, 2, 3}, {1, 1, 1, 2}, {1, 2, 4, 3}, 1);
   // Write second fragment.
@@ -1877,13 +1877,13 @@ TEST_CASE_METHOD(
   create_sparse_array(true);
 
   SECTION("no serialization") {
-    serialized_writes = false;
+    serialized_writes_ = false;
   }
-  SECTION("serialization enabled global order write") {
 #ifdef TILEDB_SERIALIZATION
-    serialized_writes = true;
-#endif
+  SECTION("serialization enabled global order write") {
+    serialized_writes_ = true;
   }
+#endif
   // Write first fragment.
   write_sparse({0, 1, 2, 3}, {1, 1, 1, 2}, {1, 2, 4, 3}, 1);
   // Write second fragment.
