@@ -31,7 +31,7 @@
  */
 
 #include <test/support/tdb_catch.h>
-#include "test/src/helpers.h"
+#include "test/support/src/helpers.h"
 #include "tiledb/sm/c_api/tiledb.h"
 #include "tiledb/sm/cpp_api/tiledb"
 
@@ -56,7 +56,7 @@ struct StringEmptyFx {
   void create_array(const std::string& array_name);
   void delete_array(const std::string& array_name);
   void read_array(const std::string& array_name);
-  void write_array(const std::string& array_name);
+  void write_array(const std::string& array_name, const bool serialized_writes);
 };
 
 // Create a simple dense 1D array with three string attributes
@@ -153,7 +153,8 @@ void StringEmptyFx::create_array(const std::string& array_name) {
   tiledb_ctx_free(&ctx);
 }
 
-void StringEmptyFx::write_array(const std::string& array_name) {
+void StringEmptyFx::write_array(
+    const std::string& array_name, const bool serialized_writes) {
   // Create TileDB context
   tiledb_ctx_t* ctx;
   int rc = tiledb_ctx_alloc(nullptr, &ctx);
@@ -250,15 +251,6 @@ void StringEmptyFx::write_array(const std::string& array_name) {
       ctx, query, "a4", buffer_a4_offsets, &buffer_a4_offsets_size);
   REQUIRE(rc == TILEDB_OK);
 
-  bool serialized_writes = false;
-  SECTION("no serialization") {
-    serialized_writes = false;
-  }
-  SECTION("serialization enabled global order write") {
-#ifdef TILEDB_SERIALIZATION
-    serialized_writes = true;
-#endif
-  }
   if (!serialized_writes) {
     rc = tiledb_query_submit(ctx, query);
     CHECK(rc == TILEDB_OK);
@@ -468,9 +460,20 @@ void StringEmptyFx::delete_array(const std::string& array_name) {
 TEST_CASE_METHOD(
     StringEmptyFx, "C API: Test empty support", "[capi][empty-var-length]") {
   std::string array_name = "empty_string";
+
+  bool serialized_writes = false;
+  SECTION("no serialization") {
+    serialized_writes = false;
+  }
+#ifdef TILEDB_SERIALIZATION
+  SECTION("serialization enabled global order write") {
+    serialized_writes = true;
+  }
+#endif
+
   delete_array(array_name);
   create_array(array_name);
-  write_array(array_name);
+  write_array(array_name, serialized_writes);
   read_array(array_name);
   delete_array(array_name);
 }
