@@ -228,6 +228,9 @@ const std::string write_file_suffix = ".wrt";
 /** Suffix for the special delete files used in TileDB. */
 const std::string delete_file_suffix = ".del";
 
+/** Suffix for the special update files used in TileDB. */
+const std::string update_file_suffix = ".upd";
+
 /** Suffix for the special metadata files used in TileDB. */
 const std::string meta_file_suffix = ".meta";
 
@@ -395,6 +398,12 @@ const std::string filter_scale_float_str = "SCALE_FLOAT";
 /** String describing FILTER_XOR. */
 const std::string filter_xor_str = "XOR";
 
+/** String describing FILTER_BITSORT. */
+const std::string filter_bitsort_str = "BITSORT";
+
+/** String describing FILTER_WEBP. */
+const std::string filter_webp_str = "WEBP";
+
 /** The string representation for FilterOption type compression_level. */
 const std::string filter_option_compression_level_str = "COMPRESSION_LEVEL";
 
@@ -416,6 +425,15 @@ const std::string filter_option_scale_float_factor = "SCALE_FLOAT_FACTOR";
 /** The string representation for FilterOption type scale_float_offset. */
 const std::string filter_option_scale_float_offset = "SCALE_FLOAT_OFFSET";
 
+/** The string representation for FilterOption type webp_quality. */
+const std::string filter_option_webp_quality = "WEBP_QUALITY";
+
+/** The string representation for FilterOption type webp_input_format. */
+const std::string filter_option_webp_input_format = "WEBP_INPUT_FORMAT";
+
+/** The string representation for FilterOption type webp_lossless. */
+const std::string filter_option_webp_lossless = "WEBP_LOSSLESS";
+
 /** The string representation for type int32. */
 const std::string int32_str = "INT32";
 
@@ -619,7 +637,7 @@ const int32_t library_version[3] = {
     TILEDB_VERSION_MAJOR, TILEDB_VERSION_MINOR, TILEDB_VERSION_PATCH};
 
 /** The TileDB serialization base format version number. */
-const uint32_t base_format_version = 16;
+const format_version_t base_format_version = 16;
 
 /**
  * The TileDB serialization format version number.
@@ -627,22 +645,22 @@ const uint32_t base_format_version = 16;
  * Conditionally set the high bit on the base_format_version to
  * easily identify that the build is experimental.
  **/
-const uint32_t format_version =
+const format_version_t format_version =
     is_experimental_build ?
         0b10000000000000000000000000000000 | base_format_version :
         base_format_version;
 
 /** The lowest version supported for back compat writes. */
-const uint32_t back_compat_writes_min_format_version = 7;
+const format_version_t back_compat_writes_min_format_version = 7;
 
 /** The lowest version supported for consolidation with timestamps. */
-const uint32_t consolidation_with_timestamps_min_version = 15;
+const format_version_t consolidation_with_timestamps_min_version = 15;
 
 /** The lowest version supported for deletes. */
-const uint32_t deletes_min_version = 16;
+const format_version_t deletes_min_version = 16;
 
 /** The lowest version supported for updates. */
-const uint32_t updates_min_version = 16;
+const format_version_t updates_min_version = 16;
 
 /** The maximum size of a tile chunk (unit of compression) in bytes. */
 const uint64_t max_tile_chunk_size = 64 * 1024;
