@@ -36,14 +36,17 @@
 #include "tiledb/common/logger.h"
 #include "tiledb/common/memory.h"
 #include "tiledb/sm/array/array.h"
+#include "tiledb/sm/array_schema/dimension_label_reference.h"
 #include "tiledb/sm/enums/query_status.h"
 #include "tiledb/sm/enums/query_type.h"
 #include "tiledb/sm/fragment/fragment_metadata.h"
 #include "tiledb/sm/misc/parse_argument.h"
-#include "tiledb/sm/query/deletes_and_updates/deletes.h"
+#include "tiledb/sm/query/deletes_and_updates/deletes_and_updates.h"
+#include "tiledb/sm/query/dimension_label/array_dimension_label_queries.h"
 #include "tiledb/sm/query/legacy/reader.h"
 #include "tiledb/sm/query/query_condition.h"
 #include "tiledb/sm/query/readers/dense_reader.h"
+#include "tiledb/sm/query/readers/ordered_dim_label_reader.h"
 #include "tiledb/sm/query/readers/sparse_global_order_reader.h"
 #include "tiledb/sm/query/readers/sparse_unordered_with_dups_reader.h"
 #include "tiledb/sm/query/writers/global_order_writer.h"
@@ -82,6 +85,7 @@ Query::Query(
     , storage_manager_(storage_manager)
     , stats_(storage_manager_->stats()->create_child("Query"))
     , logger_(storage_manager->logger()->clone("Query", ++logger_id_))
+    , dim_label_queries_(nullptr)
     , has_coords_buffer_(false)
     , has_zipped_coords_buffer_(false)
     , coord_buffer_is_set_(false)
@@ -93,7 +97,10 @@ Query::Query(
     , consolidation_with_timestamps_(false)
     , force_legacy_reader_(false)
     , fragment_name_(fragment_name)
-    , remote_query_(false) {
+    , remote_query_(false)
+    , is_dimension_label_ordered_read_(false)
+    , dimension_label_increasing_(true)
+    , fragment_size_(std::numeric_limits<uint64_t>::max()) {
   assert(array->is_open());
 
   subarray_ = Subarray(array_, layout_, stats_, logger_);
@@ -113,7 +120,7 @@ Query::Query(
     config_ = storage_manager->config();
 
   // Set initial subarray configuration
-  subarray_.set_config(config_);
+  throw_if_not_ok(subarray_.set_config(config_));
 
   rest_scratch_ = make_shared<Buffer>(HERE());
 }
@@ -177,7 +184,8 @@ Status Query::add_range(
     if (read_range_oob != "error" && read_range_oob != "warn")
       return logger_->status(Status_QueryError(
           "Invalid value " + read_range_oob +
-          " for sm.read_range_obb. Acceptable values are 'error' or 'warn'."));
+          " for sm.read_range_obb. Acceptable values are 'error' or "
+          "'warn'."));
 
     read_range_oob_error = read_range_oob == "error";
   } else {
@@ -237,8 +245,7 @@ Status Query::add_range_var(
         " for sm.read_range_obb. Acceptable values are 'error' or 'warn'."));
 
   // Add range
-  Range r;
-  r.set_range_var(start, start_size, end, end_size);
+  Range r{start, start_size, end, end_size};
   return subarray_.add_range(dim_idx, std::move(r), read_range_oob == "error");
 }
 
@@ -297,7 +304,8 @@ Status Query::get_range_var(
 
   uint64_t start_size = 0;
   uint64_t end_size = 0;
-  subarray_.get_range_var_size(dim_idx, range_idx, &start_size, &end_size);
+  throw_if_not_ok(
+      subarray_.get_range_var_size(dim_idx, range_idx, &start_size, &end_size));
 
   const void* range_start;
   const void* range_end;
@@ -941,6 +949,76 @@ Status Query::get_data_buffer(
   return Status::Ok();
 }
 
+void Query::get_label_data_buffer(
+    const std::string& name, void** buffer, uint64_t** buffer_size) const {
+  // Check query type
+  if (type_ != QueryType::READ && type_ != QueryType::WRITE) {
+    throw StatusException(
+        Status_QueryError("Cannot get buffer; Unsupported query type."));
+  }
+
+  // Check that dimension label exists
+  if (!array_schema_->is_dim_label(name)) {
+    throw StatusException(Status_QueryError(
+        std::string("Cannot set buffer; Invalid dimension label '") + name +
+        "'"));
+  }
+
+  // Return the buffer
+  auto it = label_buffers_.find(name);
+  if (it != label_buffers_.end()) {
+    if (array_schema_->dimension_label_reference(name).is_var()) {
+      *buffer = it->second.buffer_var_;
+      *buffer_size = it->second.buffer_var_size_;
+    } else {
+      *buffer = it->second.buffer_;
+      *buffer_size = it->second.buffer_size_;
+    }
+    return;
+  }
+
+  // Named buffer does not exist
+  *buffer = nullptr;
+  *buffer_size = nullptr;
+}
+
+void Query::get_label_offsets_buffer(
+    const std::string& name,
+    uint64_t** buffer_off,
+    uint64_t** buffer_off_size) const {
+  // Check query type
+  if (type_ != QueryType::READ && type_ != QueryType::WRITE) {
+    throw StatusException(Status_SerializationError(
+        "Cannot get buffer; Unsupported query type."));
+  }
+
+  // Check that dimension label exists
+  if (!array_schema_->is_dim_label(name)) {
+    throw StatusException(Status_QueryError(
+        std::string("Cannot set buffer; Invalid dimension label '") + name +
+        "'"));
+  }
+
+  // Error if it is fixed-sized
+  if (!array_schema_->dimension_label_reference(name).is_var()) {
+    throw StatusException(Status_QueryError(
+        std::string("Cannot set buffer; Input attribute/dimension '") + name +
+        "' is fixed-sized"));
+  }
+
+  // Attribute or dimension
+  auto it = label_buffers_.find(name);
+  if (it != label_buffers_.end()) {
+    *buffer_off = (uint64_t*)it->second.buffer_;
+    *buffer_off_size = it->second.buffer_size_;
+    return;
+  }
+
+  // Named buffer does not exist
+  *buffer_off = nullptr;
+  *buffer_off_size = nullptr;
+}
+
 Status Query::get_validity_buffer(
     const char* name,
     uint8_t** buffer_validity_bytemap,
@@ -1137,7 +1215,45 @@ Status Query::init() {
     }
 
     RETURN_NOT_OK(check_buffer_names());
-    RETURN_NOT_OK(create_strategy());
+
+    // Create dimension label queries and remove labels from subarray.
+    if (uses_dimension_labels()) {
+      // Check the layout is valid.
+      if (layout_ == Layout::GLOBAL_ORDER) {
+        return logger_->status(
+            Status_QueryError("Cannot init query; The global order layout is "
+                              "not supported when querying dimension labels"));
+      }
+
+      // Support for reading dimension label data from sparse arrays with
+      // multiple dimensions is not yet implemented. The data needs to be
+      // reformatted after reading to match the form of other attribute and
+      // dimension output.
+      if (!only_dim_label_query() && type_ == QueryType::READ &&
+          !array_schema_->dense() && array_schema_->dim_num() > 1 &&
+          !label_buffers_.empty()) {
+        return logger_->status(Status_QueryError(
+            "Cannot initialize query; Reading dimension label data is not yet "
+            "supported on sparse arrays with multiple dimensions."));
+      }
+
+      // Initialize the dimension label queries.
+      dim_label_queries_ = tdb_unique_ptr<ArrayDimensionLabelQueries>(tdb_new(
+          ArrayDimensionLabelQueries,
+          storage_manager_,
+          stats_->create_child("ArrayDimensionLabelQueries"),
+          array_,
+          subarray_,
+          label_buffers_,
+          buffers_,
+          fragment_name_));
+    }
+
+    // Create the query strategy if querying main array and the Subarray does
+    // not need to be updated.
+    if (!only_dim_label_query() && !subarray_.has_label_ranges()) {
+      RETURN_NOT_OK(create_strategy());
+    }
   }
 
   status_ = QueryStatus::INPROGRESS;
@@ -1190,8 +1306,45 @@ Status Query::process() {
         Status_QueryError("Cannot process query; Query is not initialized"));
   status_ = QueryStatus::INPROGRESS;
 
+  // Check if we need to process label ranges and update subarray before
+  // continuing to the main query.
+  if (dim_label_queries_ && !dim_label_queries_->completed_range_queries()) {
+    // Process the dimension label queries. Updates the subarray of this query
+    // to have the index ranges computed from the label ranges.
+    dim_label_queries_->process_range_queries(this);
+
+    // The dimension label query did not complete. For now, we are failing on
+    // this step. In the future, this may be updated to allow incomplete
+    // dimension label queries.
+    if (!dim_label_queries_->completed_range_queries()) {
+      status_ = QueryStatus::FAILED;
+      return logger_->status(
+          Status_QueryError("Cannot process query; Failed to read data from "
+                            "dimension label ranges."));
+    }
+
+    if (!only_dim_label_query()) {
+      if (strategy_ != nullptr) {
+        dynamic_cast<StrategyBase*>(strategy_.get())->stats()->reset();
+        strategy_ = nullptr;
+      }
+      throw_if_not_ok(create_strategy(true));
+    }
+  }
+
+  // Update query status.
+  status_ = QueryStatus::INPROGRESS;
+
   // Process query
-  Status st = strategy_->dowork();
+  Status st{Status::Ok()};
+  if (!only_dim_label_query()) {
+    st = strategy_->dowork();
+  }
+
+  // Process dimension label queries
+  if (dim_label_queries_) {
+    dim_label_queries_->process_data_queries();
+  }
 
   // Handle error
   if (!st.ok()) {
@@ -1199,20 +1352,22 @@ Status Query::process() {
     return st;
   }
 
-  // Check if the query is complete
-  bool completed = !strategy_->incomplete();
-
-  // Handle callback and status
-  if (completed) {
-    if (callback_ != nullptr)
+  // Check if the query is completed or not.
+  if ((only_dim_label_query() || !strategy_->incomplete()) &&
+      (!dim_label_queries_ || dim_label_queries_->completed())) {
+    // Main query and dimension label query are both completed. Handle the
+    // callback, then set status to complete.
+    if (callback_ != nullptr) {
       callback_(callback_data_);
+    }
     status_ = QueryStatus::COMPLETED;
-  } else {  // Incomplete
+  } else {
+    // Either the main query or the dimension lable query are incomplete.
     status_ = QueryStatus::INCOMPLETE;
   }
 
   return Status::Ok();
-}
+}  // namespace sm
 
 IQueryStrategy* Query::strategy(bool skip_checks_serialization) {
   if (strategy_ == nullptr) {
@@ -1235,6 +1390,11 @@ Status Query::reset_strategy_with_layout(
   return Status::Ok();
 }
 
+bool Query::uses_dimension_labels() const {
+  return !label_buffers_.empty() || subarray_.has_label_ranges() ||
+         dim_label_queries_;
+}
+
 Status Query::disable_checks_consolidation() {
   if (status_ != QueryStatus::UNINITIALIZED) {
     return logger_->status(Status_QueryError(
@@ -1275,33 +1435,39 @@ void Query::set_processed_conditions(
 Status Query::check_buffer_names() {
   if (type_ == QueryType::WRITE || type_ == QueryType::MODIFY_EXCLUSIVE) {
     // If the array is sparse, the coordinates must be provided
-    if (!array_schema_->dense() && !coords_info_.has_coords_)
-      return logger_->status(Status_WriterError(
-          "Sparse array writes expect the coordinates of the "
-          "cells to be written"));
+    if (!array_schema_->dense() && !coords_info_.has_coords_) {
+      return logger_->status(
+          Status_QueryError("Sparse array writes expect the coordinates of the "
+                            "cells to be written"));
+    }
 
     // If the layout is unordered, the coordinates must be provided
-    if (layout_ == Layout::UNORDERED && !coords_info_.has_coords_)
+    if (layout_ == Layout::UNORDERED && !coords_info_.has_coords_) {
       return logger_->status(
-          Status_WriterError("Unordered writes expect the coordinates of the "
-                             "cells to be written"));
+          Status_QueryError("Unordered writes expect the coordinates of the "
+                            "cells to be written"));
+    }
 
-    // All attributes/dimensions must be provided
-    auto expected_num = array_schema_->attribute_num();
-    expected_num += static_cast<decltype(expected_num)>(
-        buffers_.count(constants::timestamps));
-    expected_num += static_cast<decltype(expected_num)>(
-        buffers_.count(constants::delete_timestamps));
-    expected_num += static_cast<decltype(expected_num)>(
-        buffers_.count(constants::delete_condition_index));
-    expected_num += (coord_buffer_is_set_ || coord_data_buffer_is_set_ ||
-                     coord_offsets_buffer_is_set_) ?
-                        array_schema_->dim_num() :
-                        0;
-    if (buffers_.size() != expected_num)
-      return logger_->status(
-          Status_WriterError("Writes expect all attributes (and coordinates in "
-                             "the sparse/unordered case) to be set"));
+    // All attributes/dimensions must be provided unless this query is only for
+    // dimension labels.
+    if (!only_dim_label_query()) {
+      auto expected_num = array_schema_->attribute_num();
+      expected_num += static_cast<decltype(expected_num)>(
+          buffers_.count(constants::timestamps));
+      expected_num += static_cast<decltype(expected_num)>(
+          buffers_.count(constants::delete_timestamps));
+      expected_num += static_cast<decltype(expected_num)>(
+          buffers_.count(constants::delete_condition_index));
+      expected_num += (coord_buffer_is_set_ || coord_data_buffer_is_set_ ||
+                       coord_offsets_buffer_is_set_) ?
+                          array_schema_->dim_num() :
+                          0;
+      if (buffers_.size() != expected_num) {
+        return logger_->status(Status_QueryError(
+            "Writes expect all attributes (and coordinates in "
+            "the sparse/unordered case) to be set"));
+      }
+    }
   }
 
   return Status::Ok();
@@ -1362,6 +1528,7 @@ Status Query::create_strategy(bool skip_checks_serialization) {
           buffers_,
           subarray_,
           layout_,
+          fragment_size_,
           written_fragment_info_,
           disable_checks_consolidation_,
           processed_conditions_,
@@ -1374,16 +1541,27 @@ Status Query::create_strategy(bool skip_checks_serialization) {
           "Cannot create strategy; unsupported layout " + layout_str(layout_));
     }
   } else if (type_ == QueryType::READ) {
-    bool use_default = true;
     bool all_dense = true;
     for (auto& frag_md : fragment_metadata_) {
       all_dense &= frag_md->dense();
     }
 
-    if (use_refactored_sparse_unordered_with_dups_reader(
-            layout_, *array_schema_)) {
-      use_default = false;
-
+    if (is_dimension_label_ordered_read_) {
+      strategy_ = tdb_unique_ptr<IQueryStrategy>(tdb_new(
+          OrderedDimLabelReader,
+          stats_->create_child("Reader"),
+          logger_,
+          storage_manager_,
+          array_,
+          config_,
+          buffers_,
+          subarray_,
+          layout_,
+          condition_,
+          dimension_label_increasing_,
+          skip_checks_serialization));
+    } else if (use_refactored_sparse_unordered_with_dups_reader(
+                   layout_, *array_schema_)) {
       auto&& [st, non_overlapping_ranges]{Query::non_overlapping_ranges()};
       RETURN_NOT_OK(st);
 
@@ -1420,8 +1598,6 @@ Status Query::create_strategy(bool skip_checks_serialization) {
         !array_schema_->dense() &&
         (layout_ == Layout::GLOBAL_ORDER || layout_ == Layout::UNORDERED)) {
       // Using the reader for unordered queries to do deduplication.
-      use_default = false;
-
       auto&& [st, non_overlapping_ranges]{Query::non_overlapping_ranges()};
       RETURN_NOT_OK(st);
 
@@ -1456,7 +1632,6 @@ Status Query::create_strategy(bool skip_checks_serialization) {
             skip_checks_serialization));
       }
     } else if (use_refactored_dense_reader(*array_schema_, all_dense)) {
-      use_default = false;
       strategy_ = tdb_unique_ptr<IQueryStrategy>(tdb_new(
           DenseReader,
           stats_->create_child("Reader"),
@@ -1469,9 +1644,7 @@ Status Query::create_strategy(bool skip_checks_serialization) {
           layout_,
           condition_,
           skip_checks_serialization));
-    }
-
-    if (use_default) {
+    } else {
       strategy_ = tdb_unique_ptr<IQueryStrategy>(tdb_new(
           Reader,
           stats_->create_child("Reader"),
@@ -1485,9 +1658,9 @@ Status Query::create_strategy(bool skip_checks_serialization) {
           condition_,
           skip_checks_serialization));
     }
-  } else if (type_ == QueryType::DELETE) {
+  } else if (type_ == QueryType::DELETE || type_ == QueryType::UPDATE) {
     strategy_ = tdb_unique_ptr<IQueryStrategy>(tdb_new(
-        Deletes,
+        DeletesAndUpdates,
         stats_->create_child("Deletes"),
         logger_,
         storage_manager_,
@@ -1497,6 +1670,7 @@ Status Query::create_strategy(bool skip_checks_serialization) {
         subarray_,
         layout_,
         condition_,
+        update_values_,
         skip_checks_serialization));
   } else {
     return logger_->status(
@@ -1541,7 +1715,7 @@ Status Query::set_config(const Config& config) {
   // Set subarray's config for backwards compatibility
   // Users expect the query config to effect the subarray based on existing
   // behavior before subarray was exposed directly
-  subarray_.set_config(config_);
+  throw_if_not_ok(subarray_.set_config(config_));
 
   return Status::Ok();
 }
@@ -1637,11 +1811,25 @@ Status Query::set_buffer(
   has_coords_buffer_ |= is_dim;
 
   // Set attribute buffer
-  buffers_[name].set_data_buffer(buffer, buffer_size);
+  throw_if_not_ok(buffers_[name].set_data_buffer(buffer, buffer_size));
 
   return Status::Ok();
 }
 
+void Query::set_dimension_label_buffer(
+    const std::string& name, const QueryBuffer& buffer) {
+  if (buffer.buffer_var_ || buffer.buffer_var_size_) {
+    // Variable-length buffer. Set data buffer and offsets buffer.
+    throw_if_not_ok(
+        set_data_buffer(name, buffer.buffer_var_, buffer.buffer_var_size_));
+    throw_if_not_ok(set_offsets_buffer(
+        name, static_cast<uint64_t*>(buffer.buffer_), buffer.buffer_size_));
+  } else {
+    // Fixed-length buffer. Set data buffer only.
+    throw_if_not_ok(set_data_buffer(name, buffer.buffer_, buffer.buffer_size_));
+  }
+}
+
 Status Query::set_data_buffer(
     const std::string& name,
     void* const buffer,
@@ -1729,14 +1917,111 @@ Status Query::set_data_buffer(
   // Set attribute/dimension buffer on the appropriate buffer
   if (!array_schema_->var_size(name))
     // Fixed size data buffer
-    buffers_[name].set_data_buffer(buffer, buffer_size);
+    throw_if_not_ok(buffers_[name].set_data_buffer(buffer, buffer_size));
   else
     // Var sized data buffer
-    buffers_[name].set_data_var_buffer(buffer, buffer_size);
+    throw_if_not_ok(buffers_[name].set_data_var_buffer(buffer, buffer_size));
 
   return Status::Ok();
 }
 
+void Query::set_label_data_buffer(
+    const std::string& name,
+    void* const buffer,
+    uint64_t* const buffer_size,
+    const bool check_null_buffers) {
+  // Check the query type is valid.
+  if (type_ != QueryType::READ && type_ != QueryType::WRITE) {
+    throw StatusException(Status_SerializationError(
+        "Cannot set buffer; Unsupported query type."));
+  }
+
+  // Check buffer and buffer size.
+  if (check_null_buffers) {
+    if (buffer == nullptr && (type_ != QueryType::WRITE || *buffer_size != 0)) {
+      throw StatusException(
+          Status_QueryError("Cannot set buffer; " + name + " buffer is null"));
+    }
+    if (buffer_size == nullptr) {
+      throw StatusException(Status_QueryError(
+          "Cannot set buffer; " + name + " buffer size is null"));
+    }
+  }
+
+  // Check that dimension label exists.
+  if (!array_schema_->is_dim_label(name)) {
+    throw StatusException(Status_QueryError(
+        std::string("Cannot set buffer; Invalid dimension label '") + name +
+        "'"));
+  }
+
+  // Error if setting a new dimension label after initialization.
+  if (status_ != QueryStatus::UNINITIALIZED) {
+    throw StatusException(Status_QueryError(
+        std::string("Cannot set buffer for new dimension label '") + name +
+        "' after initialization"));
+  }
+
+  // Set dimension label buffer on the appropriate buffer depending if the label
+  // is fixed or variable length.
+  array_schema_->dimension_label_reference(name).is_var() ?
+      throw_if_not_ok(
+          label_buffers_[name].set_data_var_buffer(buffer, buffer_size)) :
+      throw_if_not_ok(
+          label_buffers_[name].set_data_buffer(buffer, buffer_size));
+}
+
+void Query::set_label_offsets_buffer(
+    const std::string& name,
+    uint64_t* const buffer_offsets,
+    uint64_t* const buffer_offsets_size,
+    const bool check_null_buffers) {
+  // Check the query type is valid.
+  if (type_ != QueryType::READ && type_ != QueryType::WRITE) {
+    throw StatusException(Status_SerializationError(
+        "Cannot set buffer; Unsupported query type."));
+  }
+
+  // Check for nullptrs.
+  if (check_null_buffers) {
+    if (buffer_offsets == nullptr) {
+      throw StatusException(
+          Status_QueryError("Cannot set buffer; " + name + " buffer is null"));
+    }
+
+    // Check buffer size
+    if (buffer_offsets_size == nullptr) {
+      throw StatusException(Status_QueryError(
+          "Cannot set buffer; " + name + " buffer size is null"));
+    }
+  }
+
+  // Check that dimension label exists.
+  if (!array_schema_->is_dim_label(name)) {
+    throw StatusException(Status_QueryError(
+        std::string("Cannot set buffer; Invalid dimension label '") + name +
+        "'"));
+  }
+
+  // Check the dimension labe is in fact variable length.
+  if (!array_schema_->dimension_label_reference(name).is_var()) {
+    throw StatusException(Status_QueryError(
+        std::string("Cannot set buffer; Input attribute/dimension '") + name +
+        "' is fixed-sized"));
+  }
+
+  // Check the query was not already initialized.
+  if (status_ != QueryStatus::UNINITIALIZED) {
+    throw StatusException(Status_QueryError(
+        std::string("Cannot set buffer for new attribute/dimension '") + name +
+        "' after initialization"));
+  }
+
+  // Set dimension label offsets buffers.
+  throw_if_not_ok(label_buffers_[name].set_offsets_buffer(
+      buffer_offsets, buffer_offsets_size));
+}
+
 Status Query::set_offsets_buffer(
     const std::string& name,
     uint64_t* const buffer_offsets,
@@ -1799,7 +2084,8 @@ Status Query::set_offsets_buffer(
   has_coords_buffer_ |= is_dim;
 
   // Set attribute/dimension buffer
-  buffers_[name].set_offsets_buffer(buffer_offsets, buffer_offsets_size);
+  throw_if_not_ok(
+      buffers_[name].set_offsets_buffer(buffer_offsets, buffer_offsets_size));
 
   return Status::Ok();
 }
@@ -1844,7 +2130,8 @@ Status Query::set_validity_buffer(
         "' after initialization"));
 
   // Set attribute/dimension buffer
-  buffers_[name].set_validity_buffer(std::move(validity_vector));
+  throw_if_not_ok(
+      buffers_[name].set_validity_buffer(std::move(validity_vector)));
 
   return Status::Ok();
 }
@@ -1931,8 +2218,10 @@ Status Query::set_buffer(
   }
 
   // Set attribute/dimension buffer
-  buffers_[name].set_data_var_buffer(buffer_val, buffer_val_size);
-  buffers_[name].set_offsets_buffer(buffer_off, buffer_off_size);
+  throw_if_not_ok(
+      buffers_[name].set_data_var_buffer(buffer_val, buffer_val_size));
+  throw_if_not_ok(
+      buffers_[name].set_offsets_buffer(buffer_off, buffer_off_size));
 
   return Status::Ok();
 }
@@ -2031,8 +2320,9 @@ Status Query::set_buffer(
         "' after initialization"));
 
   // Set attribute buffer
-  buffers_[name].set_data_buffer(buffer, buffer_size);
-  buffers_[name].set_validity_buffer(std::move(validity_vector));
+  throw_if_not_ok(buffers_[name].set_data_buffer(buffer, buffer_size));
+  throw_if_not_ok(
+      buffers_[name].set_validity_buffer(std::move(validity_vector)));
 
   return Status::Ok();
 }
@@ -2111,9 +2401,12 @@ Status Query::set_buffer(
         "' after initialization"));
 
   // Set attribute/dimension buffer
-  buffers_[name].set_data_var_buffer(buffer_val, buffer_val_size);
-  buffers_[name].set_offsets_buffer(buffer_off, buffer_off_size);
-  buffers_[name].set_validity_buffer(std::move(validity_vector));
+  throw_if_not_ok(
+      buffers_[name].set_data_var_buffer(buffer_val, buffer_val_size));
+  throw_if_not_ok(
+      buffers_[name].set_offsets_buffer(buffer_off, buffer_off_size));
+  throw_if_not_ok(
+      buffers_[name].set_validity_buffer(std::move(validity_vector)));
 
   return Status::Ok();
 }
@@ -2210,6 +2503,14 @@ Status Query::add_update_value(
   return Status::Ok();
 }
 
+void Query::add_index_ranges_from_label(
+    uint32_t dim_idx,
+    const bool is_point_ranges,
+    const void* start,
+    const uint64_t count) {
+  subarray_.add_index_ranges_from_label(dim_idx, is_point_ranges, start, count);
+}
+
 void Query::set_status(QueryStatus status) {
   status_ = status;
 }
@@ -2243,7 +2544,7 @@ Status Query::set_subarray(const void* subarray) {
       // Get read_range_oob config setting
       bool found = false;
       std::string read_range_oob_str =
-          config()->get("sm.read_range_oob", &found);
+          config().get("sm.read_range_oob", &found);
       assert(found);
       if (read_range_oob_str != "error" && read_range_oob_str != "warn")
         return logger_->status(Status_QueryError(
@@ -2308,11 +2609,6 @@ Status Query::set_subarray(const tiledb::sm::Subarray& subarray) {
   }
 
   // Set subarray
-  if (!subarray.is_set())
-    // Nothing useful to set here, will leave query with its current
-    // settings and consider successful.
-    return Status::Ok();
-
   auto prev_layout = subarray_.layout();
   subarray_ = subarray;
   subarray_.set_layout(prev_layout);
@@ -2340,11 +2636,7 @@ Status Query::set_subarray_unsafe(const NDRange& subarray) {
 }
 
 Status Query::check_buffers_correctness() {
-  if (type_ != QueryType::READ && type_ != QueryType::WRITE &&
-      type_ != QueryType::MODIFY_EXCLUSIVE && type_ != QueryType::DELETE) {
-    return LOG_STATUS(Status_SerializationError(
-        "Cannot check buffers; Unsupported query type."));
-  }
+  ensure_query_type_is_valid(type_);
 
   // Iterate through each attribute
   for (auto& attr : buffer_names()) {
@@ -2392,12 +2684,32 @@ Status Query::check_buffers_correctness() {
   return Status::Ok();
 }
 
+bool Query::only_dim_label_query() const {
+  // Returns true if all the following are true:
+  // 1. At most one dimension buffer is set.
+  // 2. No attribute buffers are set.
+  // 3. At least one label buffer is set.
+  return (
+      !label_buffers_.empty() &&
+      (buffers_.size() == 0 ||
+       (buffers_.size() == 1 &&
+        (coord_buffer_is_set_ || coord_data_buffer_is_set_ ||
+         coord_offsets_buffer_is_set_))));
+}
+
 Status Query::submit() {
   // Do not resubmit completed reads.
   if (type_ == QueryType::READ && status_ == QueryStatus::COMPLETED) {
     return Status::Ok();
   }
 
+  // Make sure fragment size is only set for global order.
+  if (fragment_size_ != std::numeric_limits<uint64_t>::max() &&
+      (layout_ != Layout::GLOBAL_ORDER || type_ != QueryType::WRITE)) {
+    throw StatusException(Status_QueryError(
+        "Fragment size is only supported for global order writes."));
+  }
+
   // Check attribute/dimensions buffers completeness before query submits
   RETURN_NOT_OK(check_buffers_correctness());
 
@@ -2411,7 +2723,8 @@ Status Query::submit() {
       RETURN_NOT_OK(create_strategy());
     }
 
-    // Check that input buffers are tile-aligned for remote global order writes
+    // Check that input buffers are tile-aligned for remote global order
+    // writes
     RETURN_NOT_OK(check_tile_alignment());
 
     // Check that input buffers >5mb for remote global order writes
@@ -2462,8 +2775,8 @@ QueryType Query::type() const {
   return type_;
 }
 
-const Config* Query::config() const {
-  return &config_;
+const Config& Query::config() const {
+  return config_;
 }
 
 stats::Stats* Query::stats() const {
@@ -2485,8 +2798,8 @@ bool Query::use_refactored_dense_reader(
   }
 
   // First check for legacy option
-  config_.get<bool>(
-      "sm.use_refactored_readers", &use_refactored_reader, &found);
+  throw_if_not_ok(config_.get<bool>(
+      "sm.use_refactored_readers", &use_refactored_reader, &found));
   // If the legacy/deprecated option is set use it over the new parameters
   // This facilitates backwards compatibility
   if (found) {
@@ -2513,8 +2826,8 @@ bool Query::use_refactored_sparse_global_order_reader(
   }
 
   // First check for legacy option
-  config_.get<bool>(
-      "sm.use_refactored_readers", &use_refactored_reader, &found);
+  throw_if_not_ok(config_.get<bool>(
+      "sm.use_refactored_readers", &use_refactored_reader, &found));
   // If the legacy/deprecated option is set use it over the new parameters
   // This facilitates backwards compatibility
   if (found) {
@@ -2543,8 +2856,8 @@ bool Query::use_refactored_sparse_unordered_with_dups_reader(
   }
 
   // First check for legacy option
-  config_.get<bool>(
-      "sm.use_refactored_readers", &use_refactored_reader, &found);
+  throw_if_not_ok(config_.get<bool>(
+      "sm.use_refactored_readers", &use_refactored_reader, &found));
   // If the legacy/deprecated option is set use it over the new parameters
   // This facilitates backwards compatibility
   if (found) {
@@ -2588,6 +2901,11 @@ void Query::set_remote_query() {
   remote_query_ = true;
 }
 
+void Query::set_dimension_label_ordered_read(bool increasing_order) {
+  is_dimension_label_ordered_read_ = true;
+  dimension_label_increasing_ = increasing_order;
+}
+
 /* ****************************** */
 /*          PRIVATE METHODS       */
 /* ****************************** */
