@@ -43,6 +43,7 @@
 #include "tiledb/sm/query/query_buffer.h"
 #include "tiledb/sm/query/query_macros.h"
 #include "tiledb/sm/query/strategy_base.h"
+#include "tiledb/sm/query/update_value.h"
 #include "tiledb/sm/subarray/subarray.h"
 
 #include <numeric>
@@ -92,8 +93,8 @@ SparseIndexReaderBase::SparseIndexReaderBase(
     , memory_budget_ratio_array_data_(0.1)
     , buffers_full_(false)
     , deletes_consolidation_no_purge_(
-          buffers_.count(constants::delete_timestamps) != 0) {
-  read_state_.done_adding_result_tiles_ = false;
+          buffers_.count(constants::delete_timestamps) != 0)
+    , bitsort_attribute_(array_schema_.bitsort_filter_attr()) {
   disable_cache_ = true;
 }
 
@@ -164,7 +165,8 @@ void SparseIndexReaderBase::init(bool skip_checks_serialization) {
 bool SparseIndexReaderBase::has_post_deduplication_conditions(
     FragmentMetadata& frag_meta) {
   return frag_meta.has_delete_meta() || !condition_.empty() ||
-         (!delete_conditions_.empty() && !deletes_consolidation_no_purge_);
+         (!delete_and_update_conditions_.empty() &&
+          !deletes_consolidation_no_purge_);
 }
 
 uint64_t SparseIndexReaderBase::cells_copied(
@@ -182,21 +184,17 @@ uint64_t SparseIndexReaderBase::cells_copied(
 }
 
 template <class BitmapType>
-tuple<Status, optional<std::pair<uint64_t, uint64_t>>>
-SparseIndexReaderBase::get_coord_tiles_size(
-    bool include_coords, unsigned dim_num, unsigned f, uint64_t t) {
+std::pair<uint64_t, uint64_t> SparseIndexReaderBase::get_coord_tiles_size(
+    unsigned dim_num, unsigned f, uint64_t t) {
   uint64_t tiles_size = 0;
 
   // Add the coordinate tiles size.
-  if (include_coords) {
+  if (include_coords_) {
     for (unsigned d = 0; d < dim_num; d++) {
       tiles_size += fragment_metadata_[f]->tile_size(dim_names_[d], t);
 
       if (is_dim_var_size_[d]) {
-        auto&& [st, temp] =
-            fragment_metadata_[f]->tile_var_size(dim_names_[d], t);
-        RETURN_NOT_OK_TUPLE(st, nullopt);
-        tiles_size += *temp;
+        tiles_size += fragment_metadata_[f]->tile_var_size(dim_names_[d], t);
       }
     }
   }
@@ -211,21 +209,25 @@ SparseIndexReaderBase::get_coord_tiles_size(
         fragment_metadata_[f]->cell_num(t) * constants::timestamp_size;
   }
 
+  // Compute bitsort attribute tile size.
+  if (bitsort_attribute_.has_value()) {
+    // Calculate memory consumption for this tile.
+    tiles_size += get_attribute_tile_size(bitsort_attribute_.value(), f, t);
+  }
+
   // Compute query condition tile sizes.
   uint64_t tiles_size_qc = 0;
   if (!qc_loaded_attr_names_.empty()) {
     for (auto& name : qc_loaded_attr_names_) {
       // Calculate memory consumption for this tile.
-      auto&& [st, tile_size] = get_attribute_tile_size(name, f, t);
-      RETURN_NOT_OK_TUPLE(st, nullopt);
-      tiles_size_qc += *tile_size;
+      tiles_size_qc += get_attribute_tile_size(name, f, t);
     }
   }
 
-  return {Status::Ok(), std::make_pair(tiles_size, tiles_size_qc)};
+  return std::make_pair(tiles_size, tiles_size_qc);
 }
 
-Status SparseIndexReaderBase::load_initial_data(bool include_coords) {
+Status SparseIndexReaderBase::load_initial_data() {
   if (initial_data_loaded_) {
     return Status::Ok();
   }
@@ -234,10 +236,10 @@ Status SparseIndexReaderBase::load_initial_data(bool include_coords) {
   read_state_.done_adding_result_tiles_ = false;
 
   // Load delete conditions.
-  auto&& [st, delete_conditions] =
-      storage_manager_->load_delete_conditions(*array_);
+  auto&& [st, conditions, update_values] =
+      storage_manager_->load_delete_and_update_conditions(*array_);
   RETURN_CANCEL_OR_ERROR(st);
-  delete_conditions_ = std::move(*delete_conditions);
+  delete_and_update_conditions_ = std::move(*conditions);
   bool make_timestamped_conditions = need_timestamped_conditions();
 
   if (make_timestamped_conditions) {
@@ -245,21 +247,21 @@ Status SparseIndexReaderBase::load_initial_data(bool include_coords) {
   }
 
   // Load processed conditions from fragment metadata.
-  if (delete_conditions_.size() > 0) {
-    load_processed_conditions();
+  if (delete_and_update_conditions_.size() > 0) {
+    throw_if_not_ok(load_processed_conditions());
   }
 
   // Make a list of dim/attr that will be loaded for query condition.
   if (!condition_.empty()) {
     for (auto& name : condition_.field_names()) {
-      if (!array_schema_.is_dim(name) || !include_coords) {
+      if (!array_schema_.is_dim(name) || !include_coords_) {
         qc_loaded_attr_names_set_.insert(name);
       }
     }
   }
-  for (auto delete_condition : delete_conditions_) {
-    for (auto& name : delete_condition.field_names()) {
-      if (!array_schema_.is_dim(name) || !include_coords) {
+  for (auto delete_and_update_condition : delete_and_update_conditions_) {
+    for (auto& name : delete_and_update_condition.field_names()) {
+      if (!array_schema_.is_dim(name) || !include_coords_) {
         qc_loaded_attr_names_set_.insert(name);
       }
     }
@@ -389,10 +391,10 @@ Status SparseIndexReaderBase::load_initial_data(bool include_coords) {
 }
 
 Status SparseIndexReaderBase::read_and_unfilter_coords(
-    bool include_coords, const std::vector<ResultTile*>& result_tiles) {
+    const std::vector<ResultTile*>& result_tiles) {
   auto timer_se = stats_->start_timer("read_and_unfilter_coords");
 
-  if (subarray_.is_set() || include_coords) {
+  if (include_coords_) {
     // Read and unfilter zipped coordinate tiles. Note that
     // this will ignore fragments with a version >= 5.
     std::vector<std::string> zipped_coords_names = {constants::coords};
@@ -412,7 +414,10 @@ Status SparseIndexReaderBase::read_and_unfilter_coords(
   std::vector<std::string> attr_to_load;
   attr_to_load.reserve(
       1 + deletes_consolidation_no_purge_ + use_timestamps_ +
-      qc_loaded_attr_names_.size());
+      qc_loaded_attr_names_.size() + bitsort_attribute_.has_value());
+  if (bitsort_attribute_.has_value()) {
+    attr_to_load.emplace_back(bitsort_attribute_.value());
+  }
   if (use_timestamps_) {
     attr_to_load.emplace_back(constants::timestamps);
   }
@@ -473,7 +478,7 @@ Status SparseIndexReaderBase::compute_tile_bitmaps(
               ->alloc_bitmap();
           return Status::Ok();
         });
-    RETURN_NOT_OK_ELSE(status, logger_->status(status));
+    RETURN_NOT_OK_ELSE(status, logger_->status_no_return_value(status));
   }
 
   // Process all tiles/cells in parallel.
@@ -570,7 +575,7 @@ Status SparseIndexReaderBase::compute_tile_bitmaps(
 
         return Status::Ok();
       });
-  RETURN_NOT_OK_ELSE(status, logger_->status(status));
+  RETURN_NOT_OK_ELSE(status, logger_->status_no_return_value(status));
 
   // For multiple range threads, bitmap cell count is done in a separate
   // parallel for.
@@ -585,7 +590,7 @@ Status SparseIndexReaderBase::compute_tile_bitmaps(
               ->count_cells();
           return Status::Ok();
         });
-    RETURN_NOT_OK_ELSE(status, logger_->status(status));
+    RETURN_NOT_OK_ELSE(status, logger_->status_no_return_value(status));
   }
 
   logger_->debug("Done computing tile bitmaps");
@@ -597,7 +602,8 @@ Status SparseIndexReaderBase::apply_query_condition(
     std::vector<ResultTile*>& result_tiles) {
   auto timer_se = stats_->start_timer("apply_query_condition");
 
-  if (!condition_.empty() || !delete_conditions_.empty() || use_timestamps_) {
+  if (!condition_.empty() || !delete_and_update_conditions_.empty() ||
+      use_timestamps_) {
     // Process all tiles in parallel.
     auto status = parallel_for(
         storage_manager_->compute_tp(),
@@ -653,7 +659,7 @@ Status SparseIndexReaderBase::apply_query_condition(
           }
 
           // Apply delete conditions.
-          if (!delete_conditions_.empty()) {
+          if (!delete_and_update_conditions_.empty()) {
             // Allocate delete condition idx vector if required. This vector
             // is used to store which delete condition deleted a particular
             // cell.
@@ -661,12 +667,14 @@ Status SparseIndexReaderBase::apply_query_condition(
               rt->allocate_per_cell_delete_condition_vector();
             }
 
-            for (uint64_t i = 0; i < delete_conditions_.size(); i++) {
+            for (uint64_t i = 0; i < delete_and_update_conditions_.size();
+                 i++) {
               if (!frag_meta->has_delete_meta() ||
                   frag_meta->get_processed_conditions_set().count(
-                      delete_conditions_[i].condition_marker()) == 0) {
+                      delete_and_update_conditions_[i].condition_marker()) ==
+                      0) {
                 auto delete_timestamp =
-                    delete_conditions_[i].condition_timestamp();
+                    delete_and_update_conditions_[i].condition_timestamp();
 
                 // Check the delete condition timestamp is after the fragment
                 // start.
@@ -674,13 +682,13 @@ Status SparseIndexReaderBase::apply_query_condition(
                   // Apply timestamped condition or regular condition.
                   if (!frag_meta->has_timestamps() ||
                       delete_timestamp > frag_meta->timestamp_range().second) {
-                    RETURN_NOT_OK(
-                        delete_conditions_[i].apply_sparse<BitmapType>(
-                            *(frag_meta->array_schema().get()),
-                            *rt,
-                            rt->post_dedup_bitmap()));
+                    RETURN_NOT_OK(delete_and_update_conditions_[i]
+                                      .apply_sparse<BitmapType>(
+                                          *(frag_meta->array_schema().get()),
+                                          *rt,
+                                          rt->post_dedup_bitmap()));
                   } else {
-                    RETURN_NOT_OK(timestamped_delete_conditions_[i]
+                    RETURN_NOT_OK(timestamped_delete_and_update_conditions_[i]
                                       .apply_sparse<BitmapType>(
                                           *(frag_meta->array_schema().get()),
                                           *rt,
@@ -693,7 +701,7 @@ Status SparseIndexReaderBase::apply_query_condition(
                     // the current delete condition if the cells was cleared
                     // by this condition and not any previous conditions.
                     rt->compute_per_cell_delete_condition(
-                        &delete_conditions_[i]);
+                        &delete_and_update_conditions_[i]);
                   } else {
                     // Count cells is dups are allowed as the regular bitmap was
                     // modified.
@@ -708,7 +716,7 @@ Status SparseIndexReaderBase::apply_query_condition(
 
           return Status::Ok();
         });
-    RETURN_NOT_OK_ELSE(status, logger_->status(status));
+    RETURN_NOT_OK_ELSE(status, logger_->status_no_return_value(status));
   }
 
   logger_->debug("Done applying query condition");
@@ -850,12 +858,12 @@ void SparseIndexReaderBase::remove_result_tile_range(uint64_t f) {
 }
 
 // Explicit template instantiations
-template tuple<Status, optional<std::pair<uint64_t, uint64_t>>>
+template std::pair<uint64_t, uint64_t>
 SparseIndexReaderBase::get_coord_tiles_size<uint64_t>(
-    bool, unsigned, unsigned, uint64_t);
-template tuple<Status, optional<std::pair<uint64_t, uint64_t>>>
+    unsigned, unsigned, uint64_t);
+template std::pair<uint64_t, uint64_t>
 SparseIndexReaderBase::get_coord_tiles_size<uint8_t>(
-    bool, unsigned, unsigned, uint64_t);
+    unsigned, unsigned, uint64_t);
 template Status SparseIndexReaderBase::apply_query_condition<
     UnorderedWithDupsResultTile<uint64_t>,
     uint64_t>(std::vector<ResultTile*>&);
