@@ -87,6 +87,7 @@ SparseUnorderedWithDupsReader<BitmapType>::SparseUnorderedWithDupsReader(
           subarray,
           layout,
           condition) {
+  include_coords_ = subarray_.is_set() || bitsort_attribute_.has_value();
   SparseIndexReaderBase::init(skip_checks_serialization);
 
   // Initialize memory budget variables.
@@ -184,7 +185,7 @@ Status SparseUnorderedWithDupsReader<BitmapType>::dowork() {
 
   // Load initial data, if not loaded already. Coords are only included if the
   // subarray is set.
-  RETURN_NOT_OK(load_initial_data(subarray_.is_set()));
+  RETURN_NOT_OK(load_initial_data());
 
   // Attributes names to process.
   std::vector<std::string> names;
@@ -200,7 +201,7 @@ Status SparseUnorderedWithDupsReader<BitmapType>::dowork() {
     stats_->add_counter("loop_num", 1);
 
     // Create the result tiles we are going to process.
-    RETURN_NOT_OK(create_result_tiles());
+    create_result_tiles();
 
     // No more tiles to process, done.
     if (result_tiles_.empty()) {
@@ -223,8 +224,7 @@ Status SparseUnorderedWithDupsReader<BitmapType>::dowork() {
 
     if (!result_tiles_created.empty()) {
       // Read and unfilter coords.
-      RETURN_NOT_OK(
-          read_and_unfilter_coords(subarray_.is_set(), result_tiles_created));
+      RETURN_NOT_OK(read_and_unfilter_coords(result_tiles_created));
 
       // Compute the tile bitmaps.
       RETURN_NOT_OK(compute_tile_bitmaps<BitmapType>(result_tiles_created));
@@ -297,32 +297,29 @@ void SparseUnorderedWithDupsReader<BitmapType>::reset() {
 }
 
 template <class BitmapType>
-tuple<Status, optional<std::pair<uint64_t, uint64_t>>>
+std::pair<uint64_t, uint64_t>
 SparseUnorderedWithDupsReader<BitmapType>::get_coord_tiles_size(
     unsigned dim_num, unsigned f, uint64_t t) {
-  auto&& [st, tiles_sizes] =
-      SparseIndexReaderBase::get_coord_tiles_size<BitmapType>(
-          subarray_.is_set(), dim_num, f, t);
-  RETURN_NOT_OK_TUPLE(st, nullopt);
+  auto tiles_sizes =
+      SparseIndexReaderBase::get_coord_tiles_size<BitmapType>(dim_num, f, t);
 
   auto frag_meta = fragment_metadata_[f];
 
   // Add the result tile structure size.
-  tiles_sizes->first += sizeof(UnorderedWithDupsResultTile<BitmapType>);
+  tiles_sizes.first += sizeof(UnorderedWithDupsResultTile<BitmapType>);
 
   // Add the tile bitmap size if there is a subarray or any condition to
   // process.
   if (subarray_.is_set() || has_post_deduplication_conditions(*frag_meta) ||
       process_partial_timestamps(*frag_meta)) {
-    tiles_sizes->first += frag_meta->cell_num(t) * sizeof(BitmapType);
+    tiles_sizes.first += frag_meta->cell_num(t) * sizeof(BitmapType);
   }
 
-  return {Status::Ok(), *tiles_sizes};
+  return tiles_sizes;
 }
 
 template <class BitmapType>
-tuple<Status, optional<bool>>
-SparseUnorderedWithDupsReader<BitmapType>::add_result_tile(
+bool SparseUnorderedWithDupsReader<BitmapType>::add_result_tile(
     const unsigned dim_num,
     const uint64_t memory_budget_qc_tiles,
     const uint64_t memory_budget_coords_tiles,
@@ -331,15 +328,14 @@ SparseUnorderedWithDupsReader<BitmapType>::add_result_tile(
     const uint64_t last_t,
     const FragmentMetadata& frag_md) {
   // Calculate memory consumption for this tile.
-  auto&& [st, tiles_sizes] = get_coord_tiles_size(dim_num, f, t);
-  RETURN_NOT_OK_TUPLE(st, nullopt);
-  auto tiles_size = tiles_sizes->first;
-  auto tiles_size_qc = tiles_sizes->second;
+  auto tiles_sizes = get_coord_tiles_size(dim_num, f, t);
+  auto tiles_size = tiles_sizes.first;
+  auto tiles_size_qc = tiles_sizes.second;
 
   // Don't load more tiles than the memory budget.
   if (memory_used_for_coords_total_ + tiles_size > memory_budget_coords_tiles ||
       memory_used_qc_tiles_total_ + tiles_size_qc > memory_budget_qc_tiles) {
-    return {Status::Ok(), true};
+    return true;
   }
 
   // Adjust memory usage.
@@ -353,11 +349,11 @@ SparseUnorderedWithDupsReader<BitmapType>::add_result_tile(
   if (t == last_t)
     all_tiles_loaded_[f] = true;
 
-  return {Status::Ok(), false};
+  return false;
 }
 
 template <class BitmapType>
-Status SparseUnorderedWithDupsReader<BitmapType>::create_result_tiles() {
+void SparseUnorderedWithDupsReader<BitmapType>::create_result_tiles() {
   auto timer_se = stats_->start_timer("create_result_tiles");
 
   // For easy reference.
@@ -383,7 +379,7 @@ Status SparseUnorderedWithDupsReader<BitmapType>::create_result_tiles() {
 
           // Add all tiles for this range.
           for (uint64_t t = range.first; t <= range.second; t++) {
-            auto&& [st, exceeded] = add_result_tile(
+            budget_exceeded = add_result_tile(
                 dim_num,
                 memory_budget_qc_tiles,
                 memory_budget_coords,
@@ -391,19 +387,16 @@ Status SparseUnorderedWithDupsReader<BitmapType>::create_result_tiles() {
                 t,
                 last_t,
                 *fragment_metadata_[f]);
-            RETURN_NOT_OK(st);
 
             // Make sure we can add at least one tile.
-            if (*exceeded) {
+            if (budget_exceeded) {
               logger_->debug(
                   "Budget exceeded adding result tiles, fragment {0}, tile {1}",
                   f,
                   t);
               if (result_tiles_.empty())
-                return logger_->status(
-                    Status_SparseUnorderedWithDupsReaderError(
-                        "Cannot load a single tile, increase memory budget"));
-              budget_exceeded = true;
+                throw SparseUnorderedWithDupsReaderStatusException(
+                    "Cannot load a single tile, increase memory budget");
               break;
             }
 
@@ -434,7 +427,7 @@ Status SparseUnorderedWithDupsReader<BitmapType>::create_result_tiles() {
 
         // Add all tiles for this fragment.
         for (uint64_t t = start; t < tile_num; t++) {
-          auto&& [st, exceeded] = add_result_tile(
+          budget_exceeded = add_result_tile(
               dim_num,
               memory_budget_qc_tiles,
               memory_budget_coords,
@@ -442,17 +435,15 @@ Status SparseUnorderedWithDupsReader<BitmapType>::create_result_tiles() {
               t,
               tile_num - 1,
               *fragment_metadata_[f]);
-          (void)st;
           // Make sure we can add at least one tile.
-          if (*exceeded) {
+          if (budget_exceeded) {
             logger_->debug(
                 "Budget exceeded adding result tiles, fragment {0}, tile {1}",
                 f,
                 t);
             if (result_tiles_.empty())
-              return logger_->status(Status_SparseUnorderedWithDupsReaderError(
-                  "Cannot load a single tile, increase memory budget"));
-            budget_exceeded = true;
+              throw SparseUnorderedWithDupsReaderStatusException(
+                  "Cannot load a single tile, increase memory budget");
             break;
           }
         }
@@ -475,7 +466,6 @@ Status SparseUnorderedWithDupsReader<BitmapType>::create_result_tiles() {
   }
 
   read_state_.done_adding_result_tiles_ = done_adding_result_tiles;
-  return Status::Ok();
 }
 
 template <class BitmapType>
@@ -802,7 +792,7 @@ Status SparseUnorderedWithDupsReader<BitmapType>::copy_offsets_tiles(
 
         return Status::Ok();
       });
-  RETURN_NOT_OK_ELSE(status, logger_->status(status));
+  RETURN_NOT_OK_ELSE(status, logger_->status_no_return_value(status));
 
   return Status::Ok();
 }
@@ -897,7 +887,7 @@ Status SparseUnorderedWithDupsReader<BitmapType>::copy_var_data_tiles(
 
         return Status::Ok();
       });
-  RETURN_NOT_OK_ELSE(status, logger_->status(status));
+  RETURN_NOT_OK_ELSE(status, logger_->status_no_return_value(status));
 
   return Status::Ok();
 }
@@ -1304,7 +1294,7 @@ Status SparseUnorderedWithDupsReader<BitmapType>::copy_fixed_data_tiles(
 
         return Status::Ok();
       });
-  RETURN_NOT_OK_ELSE(status, logger_->status(status));
+  RETURN_NOT_OK_ELSE(status, logger_->status_no_return_value(status));
 
   return Status::Ok();
 }
@@ -1411,11 +1401,17 @@ SparseUnorderedWithDupsReader<BitmapType>::respect_copy_memory_budget(
 
         // For dimensions, when we have a subarray, tiles are already all
         // loaded in memory.
-        if ((subarray_.is_set() && array_schema_.is_dim(name)) ||
+        if ((include_coords_ && array_schema_.is_dim(name)) ||
             qc_loaded_attr_names_set_.count(name) != 0 || is_timestamps) {
           return Status::Ok();
         }
 
+        // Bitsort attribute is already loaded in memory.
+        if (bitsort_attribute_.has_value() &&
+            name == bitsort_attribute_.value()) {
+          return Status::Ok();
+        }
+
         // Get the size for all tiles.
         uint64_t idx = 0;
         for (; idx < max_rt_idx; idx++) {
@@ -1428,23 +1424,22 @@ SparseUnorderedWithDupsReader<BitmapType>::respect_copy_memory_budget(
             continue;
           }
 
-          auto&& [st, tile_size] =
+          auto tile_size =
               get_attribute_tile_size(name, rt->frag_idx(), rt->tile_idx());
-          RETURN_NOT_OK(st);
 
           // Account for the pointers to the var data that is created in
           // copy_tiles for var sized attributes.
           if (var_sized) {
-            *tile_size += sizeof(void*) * rt->result_num();
+            tile_size += sizeof(void*) * rt->result_num();
           }
 
           // Stop when we reach the budget.
-          if (*mem_usage + *tile_size > memory_budget) {
+          if (*mem_usage + tile_size > memory_budget) {
             break;
           }
 
           // Adjust memory usage.
-          *mem_usage += *tile_size;
+          *mem_usage += tile_size;
         }
 
         // Save the minimum result tile index that we saw for all attributes.
@@ -1455,7 +1450,8 @@ SparseUnorderedWithDupsReader<BitmapType>::respect_copy_memory_budget(
 
         return Status::Ok();
       });
-  RETURN_NOT_OK_ELSE_TUPLE(status, logger_->status(status), nullopt);
+  RETURN_NOT_OK_ELSE_TUPLE(
+      status, logger_->status_no_return_value(status), nullopt);
 
   if (max_rt_idx == 0)
     return {Status_SparseUnorderedWithDupsReaderError(
@@ -1652,8 +1648,12 @@ Status SparseUnorderedWithDupsReader<BitmapType>::process_tiles(
         for (const auto& idx : *index_to_copy) {
           const auto& name_to_clear = names[idx];
           const auto is_dim_to_clear = array_schema_.is_dim(name_to_clear);
-          if (qc_loaded_attr_names_set_.count(name_to_clear) == 0 &&
-              (!subarray_.is_set() || !is_dim_to_clear)) {
+          const auto is_bitsort_attr =
+              bitsort_attribute_.has_value() &&
+              bitsort_attribute_.value() == name_to_clear;
+          if (!is_bitsort_attr &&
+              qc_loaded_attr_names_set_.count(name_to_clear) == 0 &&
+              (!include_coords_ || !is_dim_to_clear)) {
             clear_tiles(name_to_clear, result_tiles, new_result_tiles_size);
           }
         }
@@ -1689,8 +1689,10 @@ Status SparseUnorderedWithDupsReader<BitmapType>::process_tiles(
         *query_buffer.validity_vector_.buffer_size() = total_cells;
 
       // Clear tiles from memory.
-      if (qc_loaded_attr_names_set_.count(name) == 0 &&
-          (!subarray_.is_set() || !is_dim) && name != constants::timestamps &&
+      const auto is_bitsort_attr =
+          bitsort_attribute_.has_value() && bitsort_attribute_.value() == name;
+      if (!is_bitsort_attr && qc_loaded_attr_names_set_.count(name) == 0 &&
+          (!include_coords_ || !is_dim) && name != constants::timestamps &&
           name != constants::delete_timestamps) {
         clear_tiles(name, result_tiles);
       }
@@ -1731,11 +1733,10 @@ Status SparseUnorderedWithDupsReader<BitmapType>::remove_result_tile(
     typename std::list<UnorderedWithDupsResultTile<BitmapType>>::iterator rt) {
   // Remove coord tile size from memory budget.
   const auto tile_idx = rt->tile_idx();
-  auto&& [st, tiles_sizes] =
+  auto tiles_sizes =
       get_coord_tiles_size(array_schema_.dim_num(), frag_idx, tile_idx);
-  RETURN_NOT_OK(st);
-  auto tiles_size = tiles_sizes->first;
-  auto tiles_size_qc = tiles_sizes->second;
+  auto tiles_size = tiles_sizes.first;
+  auto tiles_size_qc = tiles_sizes.second;
 
   {
     std::unique_lock<std::mutex> lck(mem_budget_mtx_);
