@@ -250,7 +250,7 @@ Status fragment_metadata_from_capnp(
     // Whilst sparse gets its domain calculated, dense needs to have it
     // set here from the deserialized data
     if (array_schema->dense()) {
-      frag_meta->init_domain(*ndrange);
+      throw_if_not_ok(frag_meta->init_domain(*ndrange));
     } else {
       const auto& frag0_dom = *ndrange;
       frag_meta->non_empty_domain().assign(frag0_dom.begin(), frag0_dom.end());
